#pragma once

#include <map>
#include <nlohmann/json.hpp>
#include <optional>
#include <vector>

#define let const auto

struct SQLParserRes {
    int exit_code;
    nlohmann::json out;
    std::string err;
};

struct TableCol {
    std::string column_name;
    std::string type;
    std::string data_type;
    bool primary_key;
    NLOHMANN_DEFINE_TYPE_INTRUSIVE(TableCol, column_name, type, data_type,
                                   primary_key);
};

using TableCols = std::vector<TableCol>;
using std::nullopt;
using std::optional;

SQLParserRes parse_sql(const std::string& sql, const std::string& sql_parser_prefix =
                                                   "xmake run sql-parser");
class ExistTables {
   public:
    std::string table_file_name;
    std::map<std::string, TableCols> tables;

    ExistTables(const char* file_name = "./tables.json");
    ExistTables* read_from_file();
    bool exists(const std::string& table_name);
    void set(const std::string& table_name, const TableCols& cols);
    TableCols operator[](const std::string& table_name);
    TableCols operator[](const std::vector<std::string>& table_names);
    TableCols get(const std::string& table_name);
    TableCols get(const std::vector<std::string>& table_names);

    optional<TableCol> find_col_in_tables(
        const std::vector<std::string>& table_names,
        const std::string col_name);
    void remove(const std::string& table_name);
    void save();

    optional<TableCol> get_select_column_define(
        const std::vector<std::string> table_names,
        const nlohmann::json& select_col);
};

inline auto table_name_of(const nlohmann::json& j) {
    return j.value("table_name", "none");
}
inline auto table_names_of(const nlohmann::json& j) {
    // return j.value("table_names", json::array());
    return j["table_names"];
}
inline std::string type_of(const nlohmann::json& j) { return j.value("type", "none"); }
inline nlohmann::json right_value(const nlohmann::json& j) {
    return j["right"]["value"];
}

std::string select_column_get_name(const nlohmann::json& j);

bool is_column(const nlohmann::json& j);
bool is_const_column(const nlohmann::json& j);

enum OpType {
    OP_COMPARE,
    OP_LOGIC,
    OP_CONTAINS,
    OP_UNKNOWN,
};

OpType op_type_of(const nlohmann::json& j);
